@@ -50,7 +50,8 @@ package RT;
 use warnings;
 use strict;
 
-our $VERSION = '4.0.23';
+our $VERSION = '4.2.12';
+our ($MAJOR_VERSION, $MINOR_VERSION, $REVISION) = $VERSION =~ /^(\d)\.(\d)\.(\d+)/;
 
 
 
@@ -59,13 +60,16 @@ $EtcPath = '/etc/rt3';
 $BinPath = '/usr/local/bin';
 $SbinPath = '/usr/local/sbin';
 $VarPath = '/var/www/rt3';
-$LexiconPath = '';
-$PluginPath = '';
+$FontPath = '/usr/local/share/rt3/fonts';
+$LexiconPath = '/usr/local/share/rt3/po';
+$StaticPath = '/usr/local/share/rt3/static';
+$PluginPath = '/usr/local/plugins';
 $LocalPath = '/usr/local/share/rt3/local';
 $LocalEtcPath = '/etc/rt3/local';
 $LocalLibPath        =    '/usr/local/libdata/perl5/site_perl';
 $LocalLexiconPath = '/usr/local/share/rt3/local/po';
-$LocalPluginPath = '';
+$LocalStaticPath = '/usr/local/share/rt3/local/static';
+$LocalPluginPath = '/usr/local/share/rt3/local/plugins';
 # $MasonComponentRoot is where your rt instance keeps its mason html files
 $MasonComponentRoot = '/usr/local/share/rt3/html';
 # $MasonLocalComponentRoot is where your rt instance keeps its site-local
