@@ -145,6 +145,12 @@ sub resolve_local_link {
         # We process README separately in devel/tools/rt-static-docs
         $local = $name;
     }
+    elsif ($name =~ /^UPGRADING.*/) {
+        # If an UPGRADING file is referred to anywhere else (such as
+        # templates.pod) we won't have seen UPGRADING yet and will treat
+        # it as a non-local file.
+        $local = $name;
+    }
     # These matches handle links that look like filenames, such as those we
     # parse out of F<> tags.
     elsif (   $name =~ m{^(?:lib/)(RT/[\w/]+?)\.pm$}
