@@ -68,10 +68,10 @@ package RT::Templates;
 use strict;
 use warnings;
 
-use RT::Template;
-
 use base 'RT::SearchBuilder';
 
+use RT::Template;
+
 sub Table { 'Templates'}
 
 
@@ -140,18 +140,6 @@ sub AddRecord {
     return $self->SUPER::AddRecord( $record );
 }
 
-=head2 NewItem
-
-Returns an empty new RT::Template item
-
-=cut
-
-sub NewItem {
-    my $self = shift;
-    return(RT::Template->new($self->CurrentUser));
-}
-
-
 RT::Base->_ImportOverlays();
 
 1;
